from infi.pyutils.lazy import cached_method
from xml.sax.saxutils import escape, unescape
from ..base import Base
from os.path import abspath, dirname, join, pardir, exists
from os import makedirs

DATA_MODULES_DIR = abspath(join(dirname(__file__), pardir, pardir, 'do'))
HANDLERS_DIR = abspath(join(dirname(__file__), pardir, 'handlers'))

DATA_OBJECT_TYPES_TEMPLATE = """

from ..thirdparty import Enum

DataObjectTypes = Enum(*[
    {%- for name in types %}
    "{{ name }}",
    {%- endfor %}
    ])
"""

HANLDERS_IMPORT_METHOD = """
def import_handler(name):
    from pyvisdk.utils import camel_to_under
    from brownie.importing import import_string
    return import_string('{}.ha_cli_handler_{}.{}'.format(__name__, camel_to_under(name), name))
"""

HANDLER_MODULE_TEMPLATE = """
from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class {{ class_name }}(Base):
    '''
    {{ class_doc }}
    '''
    moid = {{ moid }}

    {%- for method in methods %}
    def {{ method.name }}({{ method.formatted_args }}):
        '''
        {{ method.doc }}
        {%- for kwarg in method.kwargs %}
        :param {{ kwarg.name }}: {{ kwarg.doc }}
        {%- endfor %}
        :returns: {{ method.returns_doc }}
        '''
        return execute_soap(self._client, self._host, self.moid, {{ method.wsdl_name_repr }},
                            {%- for kwarg in method.kwargs %}
                            {{ kwarg.name }}={{ kwarg.name }},
                            {%- endfor %}
                            )

    {%- endfor %}
"""

DATA_OBJECT_TEMPLATE = """
import logging
from pyvisdk.exceptions import InvalidArgumentError

# This module is NOT auto-generated
# Inspired by decompiled Java classes from vCenter's internalvim25stubs.jar
# Unless states otherside, the methods and attributes were not used by esxcli,
# and thus not tested

log = logging.getLogger(__name__)

def {{ name }}(vim, *args, **kwargs):
    obj = vim.client.factory.create('ns0:{{ name }}')

    # do some validation checking...
    if (len(args) + len(kwargs)) < {{ required_args_size }}:
        raise IndexError('Expected at least {{ required_args_size+1 }} arguments got: %d' % len(args))

    required = [ {{ required_args|join(', ') }} ]
    optional = [ {{ optional_args|join(', ') }} ]

    for name, arg in zip(required + optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj
"""


class Generator(Base):
    @cached_method
    def _get_managed_object_instances(self):
        """:returns: `DynamicTypeMgrMoInstance`"""
        HANDLER_PREFIX = "ha-cli-handler"
        return [item for item in self._get_dynamic_type_manager().DynamicTypeMgrQueryMoInstances(None)
                if item.id.startswith(HANDLER_PREFIX)]

    @cached_method
    def _get_types(self):
        """:returns: `DynamicTypeMgrAllTypeInfo`"""
        return self._get_dynamic_type_manager().DynamicTypeMgrQueryTypeInfo(None)

    @cached_method
    def _get_dynamic_type_manager(self):
        return self._host.RetrieveDynamicTypeManager()

    def _create_schema(self):
        from lxml.builder import ElementMaker
        from lxml import etree
        kwargs = {'targetNamespace': "urn:vim25",
                  'nsmap': {None: "http://www.w3.org/2001/XMLSchema",
                            'vim25':"urn:vim25",
                            'xsd':"http://www.w3.org/2001/XMLSchema",
                            'reflect':"urn:reflect"},
                  'elementFormDefault':"qualified"}
        schema = etree.Element("schema", **kwargs)
        schema.append(etree.Element("include", schemaLocation="core-types.xsd"))
        schema.append(etree.Element("include", schemaLocation="vim-types.xsd"))
        return schema

    def _fix_base(self, base):
        return base.replace("vmodl", "vim25").replace('.', ':')

    def _fix_type(self, data_type):
        BASIC_DATA_TYPES = ["string", "boolean", "long", "int", "dateTime", "short", "float" "anyType"]
        for basic_type in BASIC_DATA_TYPES:
            if data_type.startswith(basic_type):
                return "xsd:{}".format(basic_type)
        type_name = data_type.replace('[]', '')
        matching_cli_type = filter(lambda item: item.name == type_name, self._get_types().dataTypeInfo)
        if matching_cli_type:
            return "vim25:{}".format(matching_cli_type[0].wsdlName)
        return "vim25:{}".format(type_name.replace('vim.', ''))

    def _add_property_to_element(self, parent, property):
        """:param property: `DynamicTypeMgrPropertyTypeInfo`"""
        from lxml import etree
        kwargs = dict(name=property.name,
                      type=self._fix_type(property.type),
                      minOccurs="0")
        if '[]' in property.type:
            kwargs.update(dict(maxOccurs="unbounded"))
        parent.append(etree.Element("element", **kwargs))

    def _add_data_object(self, schema, data_object):
        from lxml import etree
        complex_type = etree.Element("complexType", name=data_object.wsdlName)
        complex_content = etree.Element("complexContent")
        extension = etree.Element("extension", base=self._fix_base(data_object.base[0]))
        sequence = etree.Element("sequence")
        for element in data_object.property:
            self._add_property_to_element(sequence, element)
        extension.append(sequence)
        complex_content.append(extension)
        complex_type.append(complex_content)
        schema.append(complex_type)

    @cached_method
    def generate_xsd(self):
        from lxml import etree
        schema = self._create_schema()
        for data_object in self._get_types().dataTypeInfo:
            self._add_data_object(schema, data_object)
        return schema

    def generate_xsd_to_file(self):
        from pyvisdk.client import WSDL_DIR
        from os.path import join
        from lxml import etree
        with open(join(WSDL_DIR, 'cli-types.xsd'), 'w') as fd:
            fd.write(etree.tostring(self.generate_xsd(), pretty_print=True, xml_declaration=True, encoding='UTF-8'))

    def _get_cli_info_for_motype(self, motype):
        """:returns:`VimCLIInfoInfo`"""
        from lxml import etree
        from ..executer import execute_soap
        CLI_INFO_ID = "ha-dynamic-type-manager-local-cli-cliinfo"
        CLI_FETCH_METHOD_NAME = "vim.CLIInfo.FetchCLIInfo"
        element = etree.Element("typeName")
        element.text = motype
        return execute_soap(self._client, self._host, CLI_INFO_ID, CLI_FETCH_METHOD_NAME, typeName=element)

    @cached_method
    def _get_managed_types_dict(self):
        """:returns: `DynamicTypeMgrManagedTypeInfo`"""
        return {item.name:{method.name:method for method in item.method}
                for item in self._get_types().managedTypeInfo}

    @cached_method
    def _get_handler_names_dict(self):
        return {item.moType:item.id for item in self._get_managed_object_instances()}

    def _prepare_method_parameters(self, param_type_info_list, cli_info_param_dict):
        """:param param_type_info_list: `DynamicTypeMgrParamTypeInfo`
        :param cli_info_param_dict: `VimCLIInfoParam`"""
        from munch import Munch
        return [Munch(name=param_type_info.name,
                      doc="{}, {}".format(param_type_info.type,
                                          cli_info_param_dict.get(param_type_info.name).help),
                      )
                for param_type_info in param_type_info_list]

    def prepare_method_formatted_arugments(self, param_type_info_list, cli_info_param_dict):
        """:param param_type_info_list: `DynamicTypeMgrParamTypeInfo`
        :param cli_info_param_dict: `VimCLIInfoParam`"""
        def is_optional(param_type_info):
            return any([annotation.name == 'optional' for annotation in param_type_info.annotation])

        def formatter(param_type_info):
            return "{}={}".format(param_type_info.name, 'None') if is_optional(param_type_info) else \
                    param_type_info.name

        return ', '.join(['self'] + map(formatter, filter(lambda item: not is_optional(item), param_type_info_list) +
                                        filter(lambda item: is_optional(item), param_type_info_list)))

    def _prepate_methods(self, managed_type_name, methods_dict, cli_info, handler_name):
        from munch import Munch
        methods_in_cli_info = {}
        for method in cli_info.method:
            methods_in_cli_info[method.name] = Munch(parameters={param.name: param for param in method.param},
                                                     return_value=method.ret,
                                                     help=method.help)
        methods = []
        for method in methods_dict.values():
            kwargs = self._prepare_method_parameters(method.paramTypeInfo,
                                                     methods_in_cli_info.get(method.name)['parameters'])
            formatted_args = self.prepare_method_formatted_arugments(method.paramTypeInfo,
                                                                     methods_in_cli_info.get(method.name)['parameters'])
            returns_doc = ', '.join([return_type_info.type for return_type_info in method.returnTypeInfo])
            methods.append(Munch(name=method.name,
                                 wsdl_name_repr=repr(".".join([managed_type_name, method.name.capitalize()])),
                                 doc=methods_in_cli_info.get(method.name).help,
                                 returns_doc=returns_doc,
                                 kwargs=kwargs,
                                 formatted_args=formatted_args))
        return methods

    def _render_handler_module(self, managed_type_name, methods_dict, cli_info, handler_name):
        """ Render handle module

        :param managed_type_name: string
        :param methods_dict: {name:`DynamicTypeMgrMethodTypeInfo`}
        :param cli_info: `VimCLIInfoInfo`
        :param handler_name: string
        """
        from jinja2 import Template
        template = Template(HANDLER_MODULE_TEMPLATE)
        methods = self._prepate_methods(managed_type_name, methods_dict, cli_info, handler_name)
        kwargs = dict(class_name=''.join([item.capitalize() for item in cli_info.name.split('.')]),
                      moid=repr(handler_name),
                      class_doc=cli_info.help,
                      methods=methods)
        return template.render(**kwargs)

    def _get_filename_for_handler_module(self, handler_name):
        return join(HANDLERS_DIR, '{}.py'.format(handler_name.replace('-', '_').lower()))

    def _write_handler_module_to_disk(self, handler_name, content):
        with open(self._get_filename_for_handler_module(handler_name), 'w') as fd:
            fd.write(content)

    @cached_method
    def generate_cli_handlers(self):
        methods_by_managed_types = self._get_managed_types_dict()
        handler_names = self._get_handler_names_dict()
        handlers = {}
        for managed_type_name, methods_dict in methods_by_managed_types.items():
            handler_name = handler_names.get(managed_type_name)
            if handler_name is None:
                continue
            cli_info = self._get_cli_info_for_motype(managed_type_name)
            content = self._render_handler_module(managed_type_name, methods_dict, cli_info, handler_name)
            handlers[handler_name] = content
        return handlers

    def save_cli_handlers_to_disk(self):
        if not exists(HANDLERS_DIR):
            makedirs(HANDLERS_DIR)
        with open(join(HANDLERS_DIR, '__init__.py'), 'w') as fd:
            fd.write(HANLDERS_IMPORT_METHOD)
        for name, content in self.generate_cli_handlers().items():
            self._write_handler_module_to_disk(name, content)

    @cached_method
    def generate_data_type_modules(self):
        from jinja2  import Template
        template = Template(DATA_OBJECT_TEMPLATE)
        data_type_modules = {}
        for data_object in self._get_types().dataTypeInfo:
            name = data_object.wsdlName
            required_args = [] # TODO what is required
            optional_args = [repr(property.name) for property in data_object.property]
            content = template.render(name=name, required_args=required_args,
                                      optional_args=optional_args,
                                      required_args_size=len(required_args))
            data_type_modules[name] = content
        return data_type_modules

    def _write_data_object_types_file(self):
        from pyvisdk.base import data_object_types
        from jinja2  import Template
        template = Template(DATA_OBJECT_TYPES_TEMPLATE)
        originals = set(data_object_types.DataObjectTypes.keys())
        additions = set(self.generate_data_type_modules().keys())
        types = list(originals.union(additions))
        types.sort()
        with open(str(data_object_types.__file__).replace('.pyc', '.py'), 'w') as fd:
            fd.write(template.render(types=types))

    def save_data_modules_to_disk(self):
        from pyvisdk.utils import camel_to_under
        self._write_data_object_types_file()
        for name, content in self.generate_data_type_modules().items():
            with open(join(DATA_MODULES_DIR, '{}.py'.format(camel_to_under(name))), 'w') as fd:
                fd.write(content)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualDiskDeltaDiskFormat = Enum(
    'nativeFormat',
    'redoLogFormat',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def NotFound(vim, *args, **kwargs):
    '''A NotFound error occurs when a referenced component of a managed object cannot
    be found. The referenced component can be a data object type (such as a role or
    permission) or a primitive (such as a string).For example, if the missing
    referenced component is a data object, such as VirtualSwitch, the NotFound
    error is thrown. The NotFound error is also thrown if the data object is found,
    but the referenced name (for example, "vswitch0") is not.'''

    obj = vim.client.factory.create('{urn:vim25}NotFound')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostPowerOperationType = Enum(
    'powerOff',
    'powerOn',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreDevicePartition(Base):
    '''
    Stats operations pertaining to the pluggable storage architectures' logical devices on the system. 
    '''
    moid = 'ha-cli-handler-storage-core-device-partition'
    def list(self, device=None):
        '''
        For a given device list all of the partitions
        :param device: string, Filter the output to a specific device.
        :returns: vim.EsxCLI.storage.core.device.partition.list.ScsiDevicePartition[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.device.partition.List',
                            device=device,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NetBIOSConfigInfoMode = Enum(
    'disabled',
    'enabled',
    'enabledViaDHCP',
    'unknown',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

InternetScsiSnsDiscoveryMethod = Enum(
    'isnsDhcp',
    'isnsSlp',
    'isnsStatic',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def GuestInfo(vim, *args, **kwargs):
    '''Information about the guest operating system.Most of this information is
    collected by VMware Tools. In general, be sure you have VMware Tools installed
    and that the virtual machine is running when you access this information.'''

    obj = vim.client.factory.create('{urn:vim25}GuestInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'guestState' ]
    optional = [ 'appHeartbeatStatus', 'disk', 'guestFamily', 'guestFullName', 'guestId',
        'guestOperationsReady', 'hostName', 'interactiveGuestOperationsReady',
        'ipAddress', 'ipStack', 'net', 'screen', 'toolsRunningStatus', 'toolsStatus',
        'toolsVersion', 'toolsVersionStatus', 'toolsVersionStatus2', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class TaskManager(BaseEntity):
    '''The TaskManager managed object provides an interface for creating and managing
    Task managed objects. Many operations are non-blocking, returning a Task
    managed object that can be monitored by a client application. Task managed
    objects may also be accessed through the TaskManager.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.TaskManager):
        super(TaskManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def description(self):
        '''Locale-specific, static strings that describe Task information to users.'''
        return self.update('description')
    @property
    def maxCollector(self):
        '''Maximum number of TaskHistoryCollector data objects that can exist
        concurrently, per client.'''
        return self.update('maxCollector')
    @property
    def recentTask(self):
        '''A list of Task managed objects that completed recently, that are currently
        running, or that are queued to run.'''
        return self.update('recentTask')

    
    
    def CreateCollectorForTasks(self, filter):
        '''Creates a TaskHistoryCollector, a specialized HistoryCollector that gathers
        TaskInfo data objects.Creates a TaskHistoryCollector, a specialized
        HistoryCollector that gathers TaskInfo data objects.
        
        :param filter: The specification for the task query filter.
        
        '''
        return self.delegate("CreateCollectorForTasks")(filter)
    
    def CreateTask(self, obj, taskTypeId, cancelable, initiatedBy=None, parentTaskKey=None):
        '''Creates a new Task, specifying the object with which the Task is associated,
        the type of task, and whether the task is cancelable. Use this operation in
        conjunction with the ExtensionManager.
        
        :param obj: ManagedObject with which Task will be associated
        
        :param taskTypeId: Extension registered task type identifier for type of task being created
        
        :param initiatedBy: The name of the user on whose behalf the Extension is creating the task
        
        :param cancelable: True if the task should be cancelable, else false
        
        :param parentTaskKey: Key of the task that is the parent of this taskvSphere API 4.0
        
        '''
        return self.delegate("CreateTask")(obj, taskTypeId, initiatedBy, cancelable, parentTaskKey)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.profile import Profile

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class ClusterProfile(Profile):
    ''''''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.ClusterProfile):
        super(ClusterProfile, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def UpdateClusterProfile(self, config):
        '''Update the ClusterProfile with the specified config.
        
        :param config: Specification which describes the changes.
        
        '''
        return self.delegate("UpdateClusterProfile")(config)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.host_directory_store import HostDirectoryStore

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostActiveDirectoryAuthentication(HostDirectoryStore):
    '''The HostActiveDirectoryAuthentication managed object indicates domain
    membership status and provides methods for adding a host to and removing a host
    from a domain.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostActiveDirectoryAuthentication):
        super(HostActiveDirectoryAuthentication, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def ImportCertificateForCAM_Task(self, certPath, camServer):
        '''Import the CAM server's certificate to the local store of vmwauth.Import the
        CAM server's certificate to the local store of vmwauth.
        
        :param certPath: full path of the certificate on ESXi
        
        :param camServer: IP of server providing the CAM service.
        
        '''
        return self.delegate("ImportCertificateForCAM_Task")(certPath, camServer)
    
    def JoinDomain_Task(self, domainName, userName, password):
        '''Adds the host to an Active Directory domain.Adds the host to an Active
        Directory domain.
        
        :param domainName: Name of the domain to be joined.
        
        :param userName: Name for an Active Directory account that has the authority to add hosts to the domain.
        
        :param password: Password for theaccount.
        
        '''
        return self.delegate("JoinDomain_Task")(domainName, userName, password)
    
    def JoinDomainWithCAM_Task(self, domainName, camServer):
        '''Adds the host to an Active Directory domain through CAM service.Adds the host
        to an Active Directory domain through CAM service.
        
        :param domainName: Name of the domain to be joined.
        
        :param camServer: Name of server providing the CAM service.
        
        '''
        return self.delegate("JoinDomainWithCAM_Task")(domainName, camServer)
    
    def LeaveCurrentDomain_Task(self, force):
        '''Removes the host from the Active Directory domain to which it belongs.
        
        :param force: If, any existing permissions on managed entities for Active Directory users will be deleted. Ifand such permissions exist, the operation will fail.
        
        '''
        return self.delegate("LeaveCurrentDomain_Task")(force)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostCacheConfigurationManager(BaseEntity):
    '''Solid state drive Cache Configuration Manager. This is a managed object which
    provides access to ESX performance tuning features using solid state drive
    based cache.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostCacheConfigurationManager):
        super(HostCacheConfigurationManager, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def cacheConfigurationInfo(self):
        '''The swap performance configuration for the ESX host. This includes
        configuration information for each datastore enabled for this purpose.'''
        return self.update('cacheConfigurationInfo')

    
    
    def ConfigureHostCache_Task(self, spec):
        '''Configure host cache/swap performance enhancement.
        
        :param spec: Specification for solid state drive cache configuration.
        
        '''
        return self.delegate("ConfigureHostCache_Task")(spec)



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.resource_pool import ResourcePool

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class VirtualApp(ResourcePool):
    '''Represents a multi-tiered software solution. A vApp is a collection of virtual
    machines (and potentially other vApp containers) that are operated and
    monitored as a unit. From a manage perspective, a multi-tiered vApp acts a lot
    like a virtual machine object. It has power operations, networks, datastores,
    and its resource usage can be configured.From a technical perspective, a vApp
    container is a specialized resource pool that has been extended with the
    following capabilities:Destroying a vAppWhen a vApp is destroyed, all of its
    virtual machines are destroyed, as well as any child vApps.The VApp.Delete
    privilege must be held on the vApp as well as the parent folder of the vApp.
    Also, the VApp.Delete privilege must be held on any child vApps that would be
    destroyed by the operation.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.VirtualApp):
        super(VirtualApp, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def childLink(self):
        '''List of linked children'''
        return self.update('childLink')
    @property
    def datastore(self):
        '''A collection of references to the subset of datastore objects used by this
        vApp.'''
        return self.update('datastore')
    @property
    def network(self):
        '''A collection of references to the subset of network objects that is used by
        this virtual machine.'''
        return self.update('network')
    @property
    def parentFolder(self):
        '''A reference to the parent folder in the VM and Template folder hierarchy. This
        is only set for a root vApp. A root vApp is a vApp that is not a child of
        another vApp.'''
        return self.update('parentFolder')
    @property
    def parentVApp(self):
        '''Reference to the parent vApp.'''
        return self.update('parentVApp')
    @property
    def vAppConfig(self):
        '''Configuration of this package.'''
        return self.update('vAppConfig')

    
    
    def CloneVApp_Task(self, name, target, spec):
        '''Creates a clone of this vApp.Creates a clone of this vApp.Creates a clone of
        this vApp.Creates a clone of this vApp.Creates a clone of this vApp.
        
        :param name: The name of the new vApp.
        
        :param target: The parent entity of the new vApp. Must be of type ResourcePool or VirtualApp.
        
        :param spec: Specifies how to clone the vApp.
        
        '''
        return self.delegate("CloneVApp_Task")(name, target, spec)
    
    def ExportVApp(self):
        '''Obtains an export lease on this vApp. The export lease contains a list of URLs
        for the disks of the virtual machines in this vApp, as well as a ticket that
        gives access to these URLs.Obtains an export lease on this vApp. The export
        lease contains a list of URLs for the disks of the virtual machines in this
        vApp, as well as a ticket that gives access to these URLs.
        
        '''
        return self.delegate("ExportVApp")()
    
    def PowerOffVApp_Task(self, force):
        '''Stops this vApp.Stops this vApp.Stops this vApp.
        
        :param force: If force is false, the shutdown order in the vApp is executed. If force is true, all virtual machines are powered-off (regardless of shutdown order).
        
        '''
        return self.delegate("PowerOffVApp_Task")(force)
    
    def PowerOnVApp_Task(self):
        '''Starts this vApp.Starts this vApp.Starts this vApp.Starts this vApp.
        
        '''
        return self.delegate("PowerOnVApp_Task")()
    
    def SuspendVApp_Task(self):
        '''Suspends this vApp.Suspends this vApp.Suspends this vApp.
        
        '''
        return self.delegate("SuspendVApp_Task")()
    
    def unregisterVApp_Task(self):
        '''Removes this vApp from the inventory without removing any of the virtual
        machine's files on disk. All high-level information stored with the management
        server (ESX Server or VirtualCenter) is removed, including information such as
        vApp configuration, statistics, permissions, and alarms.
        
        '''
        return self.delegate("unregisterVApp_Task")()
    
    def UpdateLinkedChildren(self, addChangeSet=None, removeSet=None):
        '''Reconfigure the set of linked children.Reconfigure the set of linked
        children.Reconfigure the set of linked children.Reconfigure the set of linked
        children.Reconfigure the set of linked children.Reconfigure the set of linked
        children.Reconfigure the set of linked children.Reconfigure the set of linked
        children.
        
        :param addChangeSet: a set of LinkInfo objects that either add a new link or modify an exisiting link.
        
        :param removeSet: a set of entities that should no longer link to this vApp.
        
        '''
        return self.delegate("UpdateLinkedChildren")(addChangeSet, removeSet)
    
    def UpdateVAppConfig(self, spec):
        '''Updates the vApp configuration.Updates the vApp configuration.
        
        :param spec: contains the updates to the current configuration. Any set element, is changed. All values in the spec that is left unset, will not be modified.
        
        '''
        return self.delegate("UpdateVAppConfig")(spec)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemVisorfs(Base):
    '''
    Operations pertaining to the visorfs memory filesytem.
    '''
    moid = 'ha-cli-handler-system-visorfs'
    def get(self):
        '''
        Obtain status information on the memory filesystem as a whole.
        :returns: vim.EsxCLI.system.visorfs.get.VisorfsStatus
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.visorfs.Get',
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class UserDirectory(BaseEntity):
    '''The UserDirectory managed object provides information about users and groups on
    a vSphere server and ESX hosts. The method RetrieveUserGroups returns a list of
    user account data. The method can perform a search operation based on specific
    criteria - user name, group name, sub-string or string matching, and, on
    Windows, domain. Use the results as input to the AuthorizationManager methods
    SetEntityPermissions and ResetEntityPermissions.The content of the returned
    results depends on the server environment:* On a Windows host,
    RetrieveUserGroups can search from the set of trusted domains on the host,
    including the primary domain of the system. A special domain (specified as an
    empty string - "") refers to the users and groups local to the host. * On an
    ESX Server or a Linux host, the search operates on the users and groups defined
    in the /etc/passwd file. Always specify an empty string ("") for the domain
    argument. If the /etc/passwd file contains Sun NIS or NIS+ users and groups,
    RetrieveUserGroups returns information about these accounts as well.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.UserDirectory):
        super(UserDirectory, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def domainList(self):
        '''List of Windows domains available for user searches. On ESX Server or Linux
        systems, this is an empty list.'''
        return self.update('domainList')

    
    
    def RetrieveUserGroups(self, searchStr, exactMatch, findUsers, findGroups, domain=None, belongsToGroup=None, belongsToUser=None):
        '''Returns a list of UserSearchResult objects describing the users and groups
        defined for the server.Returns a list of UserSearchResult objects describing
        the users and groups defined for the server.Returns a list of UserSearchResult
        objects describing the users and groups defined for the server.
        
        :param domain: Domain to be searched. If not set, then the method searches the local machine.
        
        :param searchStr: Case insensitive substring used to filter results; the search string is compared to the login and full name for users, and the name and description for groups. Leave this blank to match all users.
        
        :param belongsToGroup: If present, the returned list contains only users or groups that directly belong to the specified group. Users or groups that have indirect membership will not be included in the list.
        
        :param belongsToUser: If present, the returned list contains only groups that directly contain the specified user. Groups that indirectly contain the user will not be included in the list.
        
        :param exactMatch: Indicates the searchStr passed should match a user or group name exactly.
        
        :param findUsers: True, if users should be included in the result.
        
        :param findGroups: True, if groups should be included in the result.
        
        '''
        return self.delegate("RetrieveUserGroups")(domain, searchStr, belongsToGroup, belongsToUser, exactMatch, findUsers, findGroups)


'''
Created on Mar 6, 2011

@author: eplaster
'''
from suds import MethodNotFound
import logging
import os.path
import suds
import urllib

log = logging.getLogger(__name__)
log.setLevel(logging.INFO)

WSDL_DIR = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'wsdl')

class SudsClientFactory(object):
    _client = None
    _sms_client = None
    @classmethod
    def get_suds_client(cls):
        if cls._client is None:
            cls._client = suds.client.Client("file://" + urllib.pathname2url(os.path.join(WSDL_DIR, 'vimService.wsdl')),
                                             cachingpolicy=1, autoblend=True)
        return cls._client.clone()

    @classmethod
    def get_sms_suds_client(cls):
        if cls._sms_client is None:
            cls._sms_client = suds.client.Client("file://" + urllib.pathname2url(os.path.join(WSDL_DIR, 'smsService.wsdl')),
                                             cachingpolicy=1, autoblend=True)
        return cls._sms_client.clone()


class Client(object):
    '''
    The Client class acts as a proxy to the suds.client.Client class, in that it fixes the
    ManagedObjectReference objects and provides a more streamline interface specific to VISDK
    '''
    def __init__(self, server, timeout=90):
        '''
        Constructor
        '''
        url = "https://" + server + '/sdk'
        client = SudsClientFactory.get_suds_client()
        client.set_options(faults=True)
        client.set_options(location=url)
        client.set_options(timeout=timeout)
        self.service = client.service
        self.url = url
        self.client = client
        self.server = server

    #
    # proxying (special cases)
    #
    def __getattribute__(self, attr):
        service = super(Client, self).__getattribute__("service")

        # try the service
        try:
            _attr = getattr(service, attr)
            if _attr.__class__ == suds.client.Method:
                # we need to do something about getting to the right method here...
                return _attr
            else:
                return _attr
        except (AttributeError, MethodNotFound):
            # see if it's in the client object
            try:
                client = super(Client, self).__getattribute__("client")
                _attr = getattr(client, attr)
                return _attr
            except (AttributeError, MethodNotFound):
                # if it's a member of this class...
                return super(Client, self).__getattribute__(attr)


class SmsClient(Client):
    def __init__(self, server, timeout=90):
        '''
        Constructor
        '''
        url = "https://" + server + '/sms/sdk'
        client = SudsClientFactory.get_sms_suds_client()
        client.set_options(faults=True)
        client.set_options(location=url)
        client.set_options(timeout=timeout)
        self.service = client.service
        self.url = url
        self.client = client
        self.server = server




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostConfigManager(vim, *args, **kwargs):
    '''This data object type describes the configuration of a host across products and
    versions.'''

    obj = vim.client.factory.create('{urn:vim25}HostConfigManager')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'advancedOption', 'authenticationManager', 'autoStartManager',
        'bootDeviceSystem', 'cacheConfigurationManager', 'cpuScheduler',
        'datastoreSystem', 'dateTimeSystem', 'diagnosticSystem', 'esxAgentHostManager',
        'firewallSystem', 'firmwareSystem', 'healthStatusSystem', 'imageConfigManager',
        'iscsiManager', 'kernelModuleSystem', 'licenseManager', 'memoryManager',
        'networkSystem', 'patchManager', 'pciPassthruSystem', 'powerSystem',
        'serviceSystem', 'snmpSystem', 'storageSystem', 'virtualNicManager',
        'vmotionSystem', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineGuestState = Enum(
    'notRunning',
    'resetting',
    'running',
    'shuttingDown',
    'standby',
    'unknown',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualDeviceOption(vim, *args, **kwargs):
    '''The VirtualDeviceOption data object type contains information about a virtual
    device type, the options for configuring the virtual device, and the
    relationship between this virtual device and other devices. The vSphere API
    groups device configurations that are mutually exclusive into different
    configuration objects; each of these configuration objects may define subtypes
    for virtual device backing options that are independent of the virtual device.
    Backing-dependent options should appear in a subtype of
    VirtualDeviceBackingOption.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualDeviceOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'deprecated', 'hotRemoveSupported', 'plugAndPlay', 'type' ]
    optional = [ 'autoAssignController', 'backingOption', 'connectOption', 'controllerType',
        'defaultBackingOptionIndex', 'licensingLimit', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostUnresolvedVmfsResolutionSpecVmfsUuidResolution = Enum(
    'forceMount',
    'resignature',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VmFaultToleranceConfigIssueReasonForIssue = Enum(
    'esxAgentVm',
    'ftSecondaryVm',
    'ftUnsupportedHardware',
    'ftUnsupportedProduct',
    'haNotEnabled',
    'hasLocalDisk',
    'hasSnapshots',
    'hostInactive',
    'missingFTLoggingNic',
    'missingVMotionNic',
    'moreThanOneSecondary',
    'multipleVCPU',
    'noConfig',
    'recordReplayNotSupported',
    'replayNotSupported',
    'templateVm',
    'thinDisk',
    'verifySSLCertificateFlagNotSet',
    'video3dEnabled',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DistributedVirtualSwitchProductSpecOperationType = Enum(
    'notifyAvailableUpgrade',
    'preInstall',
    'proceedWithUpgrade',
    'updateBundleInfo',
    'upgrade',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseManagerState = Enum(
    'fault',
    'initializing',
    'marginal',
    'normal',

)



#!/usr/bin/python

# Copyright (c) 2011 Eric Plaster http://ogremountain.com/
# All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish, dis-
# tribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the fol-
# lowing conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
# ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
'''
pyvisdk - vSphere SDK  for Python
'''

from setuptools import setup, find_packages
import os.path

location = os.path.abspath(os.path.dirname(__file__))
version = open(os.path.join(location, "pyvisdk", "__init__.py")).readline().split()[-1].strip("'")

# we need to make sure we have these to python modules in our path
install_requires = ["suds-jurko", "enum", "Brownie", "lxml", "munch", "infi.pyutils", "Jinja2", "python-cjson", ]

setup(
    name = 'infi.pyvisdk',
    description = "vSphere SDK for Python",
    long_description = "vSphere SDK for Python",
    author = "Guy Rozendorn",
    author_email = "guy@rzn.co.il",
    version = version,
    url = "https://github.com/Infinidat/pyvisdk",
    license = "MIT",
    install_requires = install_requires,
    packages = ["pyvisdk", "pyvisdk.mo", "pyvisdk.base", "pyvisdk.do", "pyvisdk.enums", "pyvisdk.thirdparty", "pyvisdk.facade", "pyvisdk.facade.extension", "pyvisdk.facade.task", "pyvisdk.facade.property_collector", "pyvisdk.esxcli", "pyvisdk.esxcli.base", "pyvisdk.esxcli.executer", "pyvisdk.esxcli.generator", "pyvisdk.esxcli.handlers", ],
    package_data = {'pyvisdk': ['wsdl/*']},

    classifiers = ['Development Status :: 4 - Beta',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: MIT License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Software Development :: Libraries :: Python Modules',
                   ],
    include_package_data=True,
    )






import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterDrsVmConfigSpec(vim, *args, **kwargs):
    '''Updates the per-virtual-machine DRS configuration.To update the DRS
    configuration of a virtual machine, a copy of this object is included in the
    ClusterConfigSpecEx object passed to the method
    ReconfigureComputeResource_Task.If is used to incrementally update the cluster
    configuration (i.e., the parameter is true), then three operations are provided
    for updating the DRS configuration for a virtual machine. These operations are
    listed below (see ArrayUpdateSpec for more information on these operations).If,
    instead, this method is used to overwrite the cluster configuration (i.e., the
    parameter is false) thereby creating a new configuration, only the add
    operation is allowed. In this case, creates a DRS configuration for a virtual
    machine in the new cluster configuration.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterDrsVmConfigSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'operation' ]
    optional = [ 'info', 'removeKey', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostVMotionSystem(ExtensibleManagedObject):
    '''The VMotionSystem managed object describes the VMotion configuration of the
    host.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostVMotionSystem):
        super(HostVMotionSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def ipConfig(self):
        '''IP configuration of the VMotion VirtualNic.'''
        return self.update('ipConfig')
    @property
    def netConfig(self):
        '''VMotion network configuration.'''
        return self.update('netConfig')

    
    
    def DeselectVnic(self):
        '''Indicate that no VirtualNic should be used for VMotion.
        
        '''
        return self.delegate("DeselectVnic")()
    
    def SelectVnic(self, device):
        '''Select the VirtualNic to be used for VMotion.
        
        :param device: The device that uniquely identifies the VirtualNic.
        
        '''
        return self.delegate("SelectVnic")(device)
    
    def UpdateIpConfig(self, ipConfig):
        '''Update the IP configuration of VMotion VirtualNic.
        
        :param ipConfig: 
        
        '''
        return self.delegate("UpdateIpConfig")(ipConfig)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemVisorfsRamdisk(Base):
    '''
    Operations pertaining to visorfs ramdisks.
    '''
    moid = 'ha-cli-handler-system-visorfs-ramdisk'
    def add(self, maxsize, minsize, name, permissions, target):
        '''
        Add a new Visorfs RAM disk to the ESXi Host and mount it.
        :param maxsize: long, Maximum size (max reservation in MiB)
        :param minsize: long, Minimum size (min reservation in MiB)
        :param name: string, Name for the ramdisk
        :param permissions: string, Permissions for the root of the ramdisk (mode)
        :param target: string, Mountpoint for the ramdisk
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.visorfs.ramdisk.Add',
                            maxsize=maxsize,
                            minsize=minsize,
                            name=name,
                            permissions=permissions,
                            target=target,
                            )
    def list(self):
        '''
        List the RAM disks used by the host.
        :returns: vim.EsxCLI.system.visorfs.ramdisk.list.VisorfsRamdisk[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.visorfs.ramdisk.List',
                            )
    def remove(self, target):
        '''
        Remove a Visorfs RAM disk from the ESXi Host.
        :param target: string, Mountpoint for the ramdisk
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.visorfs.ramdisk.Remove',
                            target=target,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterFailoverLevelAdmissionControlPolicy(vim, *args, **kwargs):
    '''The ClusterFailoverLevelAdmissionControlPolicy defines the number of host
    failures that should be tolerated and still guarantee enough unfragmented
    resources to failover all powered on virtual machines on those failed
    hosts.When you use the failover level policy, vSphere HA partitions resources
    into slots. A slot represents the minimum CPU and memory resources that are
    required to support any powered-on virtual machine in the cluster.With the
    failover level policy in place, HA uses the following slot calculations to
    control virtual machine migration within the cluster:If your cluster contains
    any virtual machines that have much larger reservations than the others, they
    will distort slot size calculation. To avoid this, you can specify an upper
    bound for slot sizes; use the configuration editor in the vSphere Client to set
    the das.slotCpuInMHz and das.slotMemInMB attributes. When you use these
    attributes, there is a risk that resource fragmentation will cause virtual
    machines with resource requirements larger than the slot size to be assigned
    multiple slots. In a cluster that is close to capacity, there might be enough
    slots in aggregate for HA to successfully failover a virtual machine. However,
    if those slots are located on multiple hosts, a virtual machine assigned
    multiple slots cannot use them because a virtual machine can run on only a
    single host at a time.The CPU slot resource is the host CPU resource amount
    divided by the CPU component of the slot size; the result is rounded down. HA
    makes the same calculation for host memory resource amount. HA compares the
    results; the lower of the two numbers is the host slot capacity.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterFailoverLevelAdmissionControlPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'failoverLevel' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostNetOffloadCapabilities(vim, *args, **kwargs):
    '''Offload capabilities are used to optimize virtual machine network performance.
    When a virtual machine is transmitting on a network, some operations can be
    offloaded either to the host or to physical hardware. This data object type
    defines the set of offload capabilities that may be available on a host.This
    data object type is used both to publish the list of offload capabilities and
    to contain offload capability policy settings. The network policy logic is
    built on a two-level inheritance scheme which requires that all settings be
    optional. As a result, all properties on the NetOffloadCapabilities object must
    be optional.See HostNetworkPolicy'''

    obj = vim.client.factory.create('{urn:vim25}HostNetOffloadCapabilities')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'csumOffload', 'tcpSegmentation', 'zeroCopyXmit', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

LicenseReservationInfoState = Enum(
    'licensed',
    'noLicense',
    'notUsed',
    'unlicensedUse',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CheckTestType = Enum(
    'datastoreTests',
    'hostTests',
    'resourcePoolTests',
    'sourceTests',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemHostname(Base):
    '''
    Operations pertaining the network name of the ESX host.
    '''
    moid = 'ha-cli-handler-system-hostname'
    def set(self, domain=None, fqdn=None, host=None):
        '''
        This command allows the user to set the hostname, domain name or fully qualified domain name of the ESX host.
        :param domain: string, The domain name to set for the ESX host. This option is mutually exclusive with the --fqdn option.
        :param fqdn: string, Set the fully qualified domain name of the ESX host.
        :param host: string, The host name to set for the ESX host. This name should not contain the DNS domain name of the host and can only contain letters, numbers and '-'. NOTE this is not the fully qualified name, that can be set with the --fqdn option. This option is mutually exclusive with the --fqdn option.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.hostname.Set',
                            domain=domain,
                            fqdn=fqdn,
                            host=host,
                            )
    def get(self):
        '''
        Get the host, domain or fully qualified name of the ESX host.
        :returns: vim.EsxCLI.system.hostname.get.FullyQualifiedHostName
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.hostname.Get',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DeviceBackingNotSupported(vim, *args, **kwargs):
    '''The device is backed by a backing type which is not supported for this
    particular device.If this fault is returned as a subfault of
    DisallowedMigrationDeviceAttached, this indicates that although this backing
    for the device may be supported on the destination host, the hosts do not
    support the requested migration of the virtual machine while using this device
    with this backing.'''

    obj = vim.client.factory.create('{urn:vim25}DeviceBackingNotSupported')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'backing', 'device', 'reason', 'dynamicProperty', 'dynamicType', 'faultCause',
        'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualAppVAppState = Enum(
    'started',
    'starting',
    'stopped',
    'stopping',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SSPIAuthentication(vim, *args, **kwargs):
    '''SSPIAuthentication contains the information necessary to initiate a ticketed
    authentication session in the guest using SSPI credentials. The ticketed
    session is not stateless and stores state inside of the guest.To use
    SSPIAuthentication, populate sspiToken with a base64 encoded SSPI token. Then
    call AcquireCredentialsInGuest with the SSPIAuthentication object and no
    sessionID. After issuing the AcquireCredentialsInGuest call, a
    GuestAuthenticationChallenge will be thrown. Use the serverChallenge sspiToken
    in GuestAuthenticationChallenge to generate the proper SSPI response token.
    Populate an SSPIAuthentication object with the base64 encoded SSPI response
    token, and call AcquireCredentialsInGuest with the SSPIAuthentication object
    and the sessionID found in GuestAuthenticationChallenge.Successful
    authentication will result in a TicketedSessionAuthentication object being
    returned. You can use the TicketedSessionAuthentication in any guest operations
    function call. You should NOT attempt to use SSPIAuthentication in any guest
    operations function call.When you no longer need the
    TicketedSessionAuthentication object, you should call ReleaseCredentialsInGuest
    to free associated resources and session data.Usage notes: SSPI authentication
    has the same limitations as a duplicated primary token obtained from the
    Windows API function LogonUser with the LOGON32_LOGON_NETWORK logon type. This
    will affect programs started with StartProgramInGuest. For example, launched
    programs will be unable to use WMI functions unless the "Remote Enable"
    privilege is enabled for the user. Similarly, access to network resources may
    fail due to the limitations of the token.'''

    obj = vim.client.factory.create('{urn:vim25}SSPIAuthentication')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'sspiToken', 'interactiveSession' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostAuthenticationStore(BaseEntity):
    '''The HostAuthenticationStore base class represents both local user and host
    Active Directory authentication for an ESX host.* Local user authentication is
    always enabled. The vSphere API does not support local user configuration for a
    host. * Active Directory authentication for ESX hosts relies on an established
    Active Directory account that has the authority to add the host to a domain.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostAuthenticationStore):
        super(HostAuthenticationStore, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def info(self):
        '''Information about the authentication store.'''
        return self.update('info')

    



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterDasAamNodeStateDasState = Enum(
    'agentShutdown',
    'configuring',
    'error',
    'initialized',
    'nodeFailed',
    'running',
    'unconfiguring',
    'uninitialized',

)




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.managed_entity import ManagedEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class Datacenter(ManagedEntity):
    '''The Datacenter managed object provides the interface to the common container
    object for hosts, virtual machines, networks, and datastores. These entities
    must be under a distinct datacenter in the inventory, and datacenters may not
    be nested under other datacenters.Every Datacenter has the following set of
    dedicated folders. These folders are empty until you create entities for the
    Datacenter.* A folder for VirtualMachine, template, and VirtualApp objects. * A
    folder for a ComputeResource hierarchy. * A folder for Network,
    DistributedVirtualSwitch, and DistributedVirtualPortgroup objects. * A folder
    for Datastore objects.For a visual representation of the organization of
    objects in a vCenter hierarchy, see the description of the ServiceInstance
    object.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.Datacenter):
        super(Datacenter, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def datastore(self):
        '''A collection of references to the datastore objects available in this
        datacenter.'''
        return self.update('datastore')
    @property
    def datastoreFolder(self):
        '''A reference to the folder hierarchy that contains the datastores for this
        datacenter.'''
        return self.update('datastoreFolder')
    @property
    def hostFolder(self):
        '''A reference to the folder hierarchy that contains the compute resources,
        including hosts and clusters, for this datacenter.'''
        return self.update('hostFolder')
    @property
    def network(self):
        '''A collection of references to the network objects available in this datacenter.'''
        return self.update('network')
    @property
    def networkFolder(self):
        '''A reference to the folder hierarchy that contains the network entities for this
        datacenter. The folder can include Network, DistributedVirtualSwitch, and
        DistributedVirtualPortgroup objects.'''
        return self.update('networkFolder')
    @property
    def vmFolder(self):
        '''A reference to the folder hierarchy that contains VirtualMachine virtual
        machine templates (identified by the template property, and VirtualApp objects
        for this datacenter.'''
        return self.update('vmFolder')

    
    
    def PowerOnMultiVM_Task(self, vm, option=None):
        '''Powers on multiple virtual machines in a data center. If the virtual machines
        are suspended, this method resumes execution from the suspend point. The
        virtual machines can belong to different clusters in the data center.Powers on
        multiple virtual machines in a data center. If the virtual machines are
        suspended, this method resumes execution from the suspend point. The virtual
        machines can belong to different clusters in the data center.Powers on multiple
        virtual machines in a data center. If the virtual machines are suspended, this
        method resumes execution from the suspend point. The virtual machines can
        belong to different clusters in the data center.
        
        :param vm: The virtual machines to power on.
        
        :param option: An array of OptionValue options for this power-on session. The names and values of the options are defined in ClusterPowerOnVmOption.vSphere API 4.1
        
        '''
        return self.delegate("PowerOnMultiVM_Task")(vm, option)
    
    def QueryConnectionInfo(self, hostname, port, username, password, sslThumbprint=None):
        '''This method provides a way of getting basic information about a host without
        adding it to a datacenter. Connection wizards typically use this method to show
        information about a host so a user can confirm a set of changes before applying
        them.
        
        :param hostname: The target of the query.
        
        :param port: The port number of the target host. For ESX 2.x this is the authd port (902 by default). For ESX 3.x and above and for VMware Server hosts this is the https port (443 by default). You can specify -1 to have the vCenter Server try the default ports.
        
        :param username: The name of the user.
        
        :param password: The password of the user.
        
        :param sslThumbprint: The expected SSL thumbprint of the host's certificateVI API 2.5
        
        '''
        return self.delegate("QueryConnectionInfo")(hostname, port, username, password, sslThumbprint)



from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class IscsiAdapterDiscoveryStatus(Base):
    '''
    Operations that can be performed on iSCSI adapter discovery status
    '''
    moid = 'ha-cli-handler-iscsi-adapter-discovery-status'
    def get(self, adapter):
        '''
        Get the iSCSI adapter discovery status for the iSCSI Host Bus Adapter.
        :param adapter: string, The iSCSI adapter name.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.iscsi.adapter.discovery.status.Get',
                            adapter=adapter,
                            )   



from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostPowerSystem(BaseEntity):
    '''Managed object responsible for getting and setting host power management
    policies.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostPowerSystem):
        super(HostPowerSystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def capability(self):
        '''Power system capabilities object.'''
        return self.update('capability')
    @property
    def info(self):
        '''Power system state info object.'''
        return self.update('info')

    
    
    def ConfigurePowerPolicy(self, key):
        '''Configure host power policy.
        
        :param key: A key from one of the policies in availablePolicy.
        
        '''
        return self.delegate("ConfigurePowerPolicy")(key)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostNicOrderPolicy(vim, *args, **kwargs):
    '''This data object type describes network adapter ordering policy for a network
    adapter team. A physical network adapter can be in the active list, the standby
    list, or neither. It cannot be in both lists. For a virtual switch, the
    NicOrderPolicy property is never null when retrieved from the server. When
    creating a new virtual switch or updating an existing virtual switch, the
    NicOrderPolicy can be null, in which case the default NicOrderPolicy from the
    server will be used. For a portgroup, a null NicOrderPolicy property means the
    portgroup inherits the policy from its parent. Otherwise, the NicOrderPolicy
    property defined in the portgroup takes precedence. In all cases where the
    NicOrderPolicy property is set, an empty activeNic array means there are no
    active Ethernet adapters in the team. An empty standbyNic array means there are
    no standby Ethernet adapters.'''

    obj = vim.client.factory.create('{urn:vim25}HostNicOrderPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'activeNic', 'standbyNic', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CustomizationNetBIOSMode = Enum(
    'disableNetBIOS',
    'enableNetBIOS',
    'enableNetBIOSViaDhcp',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VAppIPAssignmentInfoIpAllocationPolicy = Enum(
    'dhcpPolicy',
    'fixedPolicy',
    'transientPolicy',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreClaiming(Base):
    '''
    Operations pertaining to the pluggable storage direct path claiming system. These operations will allow a user to directly control the claiming and unclaiming process. These operations are considered temporary and any claiming operations that need to survive a reboot should use claimrules instead.
    '''
    moid = 'ha-cli-handler-storage-core-claiming'
    def unclaim(self, type, adapter=None, channel=None, claimruleclass=None, device=None, driver=None, lun=None, model=None, path=None, plugin=None, target=None, vendor=None):
        '''
        1) Unclaim a path or set of paths, disassociating them from a PSA plugin. NOTES:  It is normal for path claiming to fail especially when unclaiming by plugin or adapter. Only inactive paths with no I/O  will be able to be unclaimed. Typically the ESXi USB partition and devices with VMFS volumes on them will not be unclaimable. Also NOTE unclaiming will not persist and periodic path claiming will reclaim these paths in the near future unless claim rules are configured to mask the path. 2) Detach a (set of) filter(s) from one or more devices.
        :param adapter: string, If the --type paramter is 'location' this value indicates the name of the host bus adapter for the paths you wish to unclaim. This parameter can be omitted to indicate unclaiming should be run on paths from all adapters.
        :param channel: long, If the --type parameter is 'location' this value indicates the value of the SCSI channel number for the paths you wish to unclaim. This parameter can be omitted to indicate unclaiming should be run on paths with any channel number.
        :param claimruleclass: string, Indicate the claim rule class to use in this operation [MP, Filter].
        :param device: string, If the --type parameter is 'device' attempt to unclaim all paths to a specific device (for multipathing plugins) or unclaim the device itself (for filter plugins). NOTE. For paths, if there are any active I/O operations on this device, at least 1 path will fail to unclaim.
        :param driver: string, If the --type parameter is 'driver' attempt to unclaim all paths provided by a specific HBA driver.
        :param lun: long, If the --type paramter is 'location' this value indicates the value of the SCSI Logical Unit Number (LUN) for the paths you wish to unclaim. This parameter can be omitted to indicate unclaiming should be run on paths with any Logical Unit Number.
        :param model: string, If the --type parameter is 'vendor' attempt to unclaim all paths to devices with specific model info (for multipathing plugins) or unclaim the device itself (for filter plugins). NOTE. For paths, if there are any active I/O operations on this device, at least 1 path will fail to unclaim.
        :param path: string, If the --type parameter is 'path' attempt to unclaim a specific path given its path UID or runtime name.
        :param plugin: string, If the --type parameter is 'plugin' attempt to unclaim all paths on for a given multipath plugin OR all devices attached to a filter plugin.
        :param target: long, If the --type paramter is 'location' this value indicates the value of the SCSI target number for the paths you wish to unclaim. This parameter can be omitted to indicate unclaiming should be run on paths with any target number.
        :param type: string, Indicate the type of unclaim you wish to perform. Valid values for this paramter are  [location, path, driver, device, plugin, vendor]
        :param vendor: string, If the --type parameter is 'vendor' attempt to unclaim all paths to devices with specific vendor info (for multipathing plugins) or unclaim the device itself (for filter plugins). NOTE. For paths, if there are any active I/O operations on this device, at least 1 path will fail to unclaim.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.claiming.Unclaim',
                            adapter=adapter,
                            channel=channel,
                            claimruleclass=claimruleclass,
                            device=device,
                            driver=driver,
                            lun=lun,
                            model=model,
                            path=path,
                            plugin=plugin,
                            target=target,
                            type=type,
                            vendor=vendor,
                            )
    def autoclaim(self, enabled, claimruleclass=None, wait=None):
        '''
        Control the automatic PSA (path/device) claiming code allowing the disabling of the automatic claiming process or re-enabling of the claiming process if it was previously disabled. By default the automatic PSA claiming process is on and should not be disabled by users unless specifically instructed to do so.
        :param claimruleclass: string, Indicate the claim rule class to use in this operation [MP, Filter, VAAI, all].
        :param enabled: boolean, Set the autoclaiming enabled state for a givenPSA plugin type in the VMkernel. Default is to have this process enabled. This should not be changed by users unless specifically instructed to do so.
        :param wait: boolean, If the --wait flag is provided then the autoclaim enable will wait for paths to 'settle' before running the claim operation.  This means that the system is reasonably sure that all paths on the system have appeared before enabling autoclaim.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.claiming.Autoclaim',
                            claimruleclass=claimruleclass,
                            enabled=enabled,
                            wait=wait,
                            )
    def reclaim(self, device):
        '''
         Attempt to unclaim all paths to a device and then run the loaded claimrules on each of the paths unclaimed to attempt to reclaim them.
        :param device: string, Reclaim requires the name of a device on which all paths will be unclaimed and then reclaimed.
        :returns: void
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.claiming.Reclaim',
                            device=device,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

PortGroupConnecteeType = Enum(
    'host',
    'systemManagement',
    'unknown',
    'virtualMachine',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostPatchManagerReason = Enum(
    'conflictLib',
    'conflictPatch',
    'hasDependentPatch',
    'missingLib',
    'missingPatch',
    'obsoleted',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualUSBRemoteHostBackingInfo(vim, *args, **kwargs):
    '''The VirtualUSBRemoteHostBackingInfo data object identifies a host and a USB
    device that is attached to the host. Use this object to configure support for
    persistent access to the USB device when vMotion operations migrate a virtual
    machine to a different host. The vCenter Server will not migrate the virtual
    machine to a host that does not support the USB remote host backing
    capability.Specify remote host backing as part of the USB device configuration
    when you create or reconfigure a virtual machine
    (VirtualMachineConfigSpec.deviceChange.device.backing).To identify the USB
    device, you specify an autoconnect pattern for the deviceName. The virtual
    machine can connect to the USB device if the ESX server can find a USB device
    described by the autoconnect pattern. The autoconnect pattern consists of
    name:value pairs. You can use any combination of the following fields.For
    example, the following pattern identifies a USB device:This pattern identifies
    the USB device connected to port 1/3/0 on the host with the unique id .Special
    characters for autoconnect pattern values:'''

    obj = vim.client.factory.create('{urn:vim25}VirtualUSBRemoteHostBackingInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'hostname', 'deviceName' ]
    optional = [ 'useAutoDetect', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ClusterDasFdmAvailabilityState = Enum(
    'connectedToMaster',
    'election',
    'fdmUnreachable',
    'hostDown',
    'initializationError',
    'master',
    'networkIsolated',
    'networkPartitionedFromMaster',
    'uninitializationError',
    'uninitialized',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostNasVolumeSpec(vim, *args, **kwargs):
    '''Specification for creating NAS volume.When mounting a NAS volume on multiple
    hosts, the same remoteHost and remotePath values should be used on every host,
    otherwise it will be treated as different datastores. For example, if one host
    references the remotePath of a NAS volume as "/mnt/mount1" and another
    references it as "/mnt/mount1/", it will not be recognized as the same
    datastore.'''

    obj = vim.client.factory.create('{urn:vim25}HostNasVolumeSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'accessMode', 'localPath', 'remoteHost', 'remotePath' ]
    optional = [ 'password', 'type', 'userName', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageNmpPspGenericDeviceconfig(Base):
    '''
    PSP device configuration operations
    '''
    moid = 'ha-cli-handler-storage-nmp-psp-generic-deviceconfig'
    def set(self, config, device):
        '''
        Allow setting of per device PSP configuration parameters.  This command will set the configuration for the given device with whichever PSP it is currently configurated with.
        :param config: string, The configuration string you wish to set.
        :param device: string, The device you wish to set PSP configuration for.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.generic.deviceconfig.Set',
                            config=config,
                            device=device,
                            )
    def get(self, device):
        '''
        Allow retrieving of per device PSP configuration parameters.
        :param device: string, The device you wish to get PSP configuration for.
        :returns: string
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.generic.deviceconfig.Get',
                            device=device,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostDiskPartitionInfoPartitionFormat = Enum(
    'gpt',
    'mbr',
    'unknown',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

RecommendationReasonCode = Enum(
    'antiAffin',
    'balanceDatastoreIOLoad',
    'balanceDatastoreSpaceUsage',
    'checkResource',
    'datastoreMaint',
    'datastoreSpaceOutage',
    'enterStandby',
    'fairnessCpuAvg',
    'fairnessMemAvg',
    'hostMaint',
    'increaseCapacity',
    'iolbDisabledInternal',
    'jointAffin',
    'powerOnVm',
    'powerSaving',
    'reservationCpu',
    'reservationMem',
    'storagePlacement',
    'unreservedCapacity',
    'virtualDiskAntiAffin',
    'virtualDiskJointAffin',
    'vmHostHardAffinity',
    'vmHostSoftAffinity',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageCoreDeviceStats(Base):
    '''
    Stats operations pertaining to the pluggable storage architectures' logical devices on the system. 
    '''
    moid = 'ha-cli-handler-storage-core-device-stats'
    def get(self, device=None):
        '''
        List the SCSI stats for SCSI Devices in the system.
        :param device: string, Limit the stats output to one specific device. This device name can be any of the UIDs the device reports
        :returns: vim.EsxCLI.storage.core.device.stats.get.ScsiDeviceStats[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.core.device.stats.Get',
                            device=device,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostListSummaryQuickStats(vim, *args, **kwargs):
    '''Basic host statistics.Included in the host statistics are fairness scores.
    Fairness scores are represented in units with relative values, meaning they are
    evaluated relative to the scores of other hosts. They should not be thought of
    as having any particular absolute value. Each fairness unit represents an
    increment of 0.001 in a fairness score. The further the fairness score diverges
    from 1, the less fair the allocation. Therefore, a fairness score of 990,
    representing 0.990, is more fair than a fairness score of 1015, which
    represents 1.015. This is because 1.015 is further from 1 than 0.990.'''

    obj = vim.client.factory.create('{urn:vim25}HostListSummaryQuickStats')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'distributedCpuFairness', 'distributedMemoryFairness', 'overallCpuUsage',
        'overallMemoryUsage', 'uptime', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def HostUnresolvedVmfsVolume(vim, *args, **kwargs):
    '''Information about detected unbound, unresolved VMFS volume. An unresolved VMFS
    volume is reported when one or more device partitions of volume are detected to
    have copies of extents of the volume. Such copies can be created via
    replication or snapshots.UnresolvedVmfsVolume are not mounted on the host where
    they are detected. User may choose to resignature the volume in which case a
    new Uuid is assigned to the volume and contents of the VMFS volume is kept
    intact.User may choose to keep the original Uuid and mount the VMFS volume as
    it is on the given host. In this case, user has choosen to mount the copy of
    the VMFS volume on that host with no change to the original Uuid. This may fail
    with VmfsVolumeAlreadyMounted exception if there is an existing VMFS volume
    with the same Uuid mounted somewhere in the same datacenter.Simple diagram
    representing the possible operations on UnresolvedVmfsVolumeSee
    HostStorageSystem'''

    obj = vim.client.factory.create('{urn:vim25}HostUnresolvedVmfsVolume')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'extent', 'resolveStatus', 'totalBlocks', 'vmfsLabel', 'vmfsUuid' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostLowLevelProvisioningManagerReloadTarget = Enum(
    'currentConfig',
    'snapshotConfig',

)




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemSyslog(Base):
    '''
    Operations relating to system logging
    '''
    moid = 'ha-cli-handler-system-syslog'
    def reload(self):
        '''
        Reload the log daemon to apply any new configuration options
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.syslog.Reload',
                            )
    def mark(self, message):
        '''
        Mark all logs with the specified string
        :param message: string, The message to place in the logs
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.syslog.Mark',
                            message=message,
                            )   



########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

NetIpStackInfoEntryType = Enum(
    'dynamic',
    'invalid',
    'manual',
    'other',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SnapshotMoveToNonHomeNotSupported(vim, *args, **kwargs):
    '''An attempt is being made to move a virtual machine's disk that has associated
    snapshots, and preserving the snapshots is not supported by the host because
    the disk is being moved to some location other than the new home datastore for
    the virtual machine.'''

    obj = vim.client.factory.create('{urn:vim25}SnapshotMoveToNonHomeNotSupported')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationIdentification(vim, *args, **kwargs):
    '''The Identification data object type provides information needed to join a
    workgroup or domain.The Identification data object type maps to the
    Identification key in the answer file. These values are transferred into the
    file that VirtualCenter stores on the target virtual disk. For more detailed
    information, see the document .'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationIdentification')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'domainAdmin', 'domainAdminPassword', 'joinDomain', 'joinWorkgroup',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class VirtualMachineProvisioningChecker(BaseEntity):
    '''A singleton managed object that can answer questions about the feasibility of
    certain provisioning operations.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.VirtualMachineProvisioningChecker):
        super(VirtualMachineProvisioningChecker, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def CheckMigrate_Task(self, vm, host=None, pool=None, state=None, testType=None):
        '''Tests the feasibility of a proposed MigrateVM_Task operation.
        
        :param vm: The virtual machine we propose to migrate.
        
        :param host: The target host on which the virtual machines will run. The host parameter may be left unset if the compute resource associated with the target pool represents a stand-alone host or a DRS-enabled cluster. In the former case the stand-alone host is used as the target host. In the latter case, each connected host in the cluster that is not in maintenance mode is tested as a target host. If the virtual machine is a template then either this parameter or the pool parameter must be set.
        
        :param pool: The target resource pool for the virtual machines. If the pool parameter is left unset, the target pool for each particular virtual machine's migration will be that virtual machine's current pool. If the virtual machine is a template then either this parameter or the host parameter must be set.
        
        :param state: The power state that the virtual machines must have. If this argument is not set, each virtual machine is evaluated according to its current power state.
        
        :param testType: The set of tests to run. If this argument is not set, all tests will be run.
        
        '''
        return self.delegate("CheckMigrate_Task")(vm, host, pool, state, testType)
    
    def CheckRelocate_Task(self, vm, spec, testType=None):
        '''Tests the feasibility of a proposed RelocateVM_Task operation.
        
        :param vm: The virtual machine we propose to relocate.
        
        :param spec: The specification of where to relocate the virtual machine. In cases where DRS would automatically select a host, all potential hosts are tested against.
        
        :param testType: 
        
        '''
        return self.delegate("CheckRelocate_Task")(vm, spec, testType)
    
    def QueryVMotionCompatibilityEx_Task(self, vm, host):
        '''Investigates the general VMotion compatibility of a set of virtual machines
        with a set of hosts. The virtual machine may be in any power state. Hosts may
        be in any connection state and also may be in maintenance mode.
        
        :param vm: The set of virtual machines to analyze for compatibility. All virtual machines are assumed to be powered-on for the purposes of this operation.
        
        :param host: The set of hosts to analyze for compatibility. All hosts are assumed to be connected and not in maintenence mode for the purposes of this operation.
        
        '''
        return self.delegate("QueryVMotionCompatibilityEx_Task")(vm, host)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DateTimeProfile(vim, *args, **kwargs):
    '''The DateTimeProfile data object represents host date and time configuration.
    Use the policy list for access to configuration data for the date and time
    profile. Use the property list for access to subprofiles, if any.'''

    obj = vim.client.factory.create('{urn:vim25}DateTimeProfile')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'enabled' ]
    optional = [ 'policy', 'profileTypeName', 'profileVersion', 'property', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkFirewallRuleset(Base):
    '''
    Commands to list and update firewall ruleset configuration
    '''
    moid = 'ha-cli-handler-network-firewall-ruleset'
    def set(self, rulesetid, allowedall=None, enabled=None):
        '''
        Set firewall ruleset status (allowedAll flag and enabled status).
        :param allowedall: boolean, Set to true to allowed all ip, set to false to use allowed ip list.
        :param enabled: boolean, Set to true to enable ruleset, set to false to disable it.
        :param rulesetid: string, The label of the ruleset.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.Set',
                            allowedall=allowedall,
                            enabled=enabled,
                            rulesetid=rulesetid,
                            )
    def list(self, rulesetid=None):
        '''
        List the rulesets in firewall.
        :param rulesetid: string, List configuration for specfic ruleset
        :returns: vim.EsxCLI.network.firewall.ruleset.list.Ruleset[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.firewall.ruleset.List',
                            rulesetid=rulesetid,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def IscsiFaultVnicIsLastPath(vim, *args, **kwargs):
    '''This fault indicates that the given Virtual NIC is associated with the only
    path to the storage. Any attempt to unbind this from iSCSI HBA would result in
    storage being inaccessible.'''

    obj = vim.client.factory.create('{urn:vim25}IscsiFaultVnicIsLastPath')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'vnicDevice', 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ObjectUpdateKind = Enum(
    'enter',
    'leave',
    'modify',

)




def import_handler(name):
    from pyvisdk.utils import camel_to_under
    from brownie.importing import import_string
    return import_string('{}.ha_cli_handler_{}.{}'.format(__name__, camel_to_under(name), name))




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def EventEx(vim, *args, **kwargs):
    '''EventEx is a dynamically typed Event class, whose type is indicated by its
    eventTypeId property.A collection of eventTypeIds is registered by Extensions,
    which can now pass in optional type information for each eventTypeId which
    indicates the applicable argument names and types, among other
    properties.EventEx allows event arguments of any type, though today, the system
    only supports "string" and "moid" (a string which can be interpreted as an
    object ID in the system) as argument types. In the future, the system may
    optionally strongly check the types of the arguments in the event against the
    declared type information, based on how the event type is declared.EventEx also
    allows arbitrary "event object"s - the object which the event refers to. You
    can put in any object identifier as the objectId, but objectType should be
    filled in only if the object is actually present in the VC Server's
    ManagedEntity inventory.'''

    obj = vim.client.factory.create('{urn:vim25}EventEx')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'eventTypeId', 'chainId', 'createdTime', 'key', 'userName' ]
    optional = [ 'arguments', 'fault', 'message', 'objectId', 'objectName', 'objectType',
        'severity', 'changeTag', 'computeResource', 'datacenter', 'ds', 'dvs',
        'fullFormattedMessage', 'host', 'net', 'vm', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

CustomizationLicenseDataMode = Enum(
    'perSeat',
    'perServer',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineGuestOsIdentifier = Enum(
    'asianux3_64Guest',
    'asianux3Guest',
    'asianux4_64Guest',
    'asianux4Guest',
    'centos64Guest',
    'centosGuest',
    'darwin10_64Guest',
    'darwin10Guest',
    'darwin11_64Guest',
    'darwin11Guest',
    'darwin64Guest',
    'darwinGuest',
    'debian4_64Guest',
    'debian4Guest',
    'debian5_64Guest',
    'debian5Guest',
    'debian6_64Guest',
    'debian6Guest',
    'dosGuest',
    'eComStation2Guest',
    'eComStationGuest',
    'freebsd64Guest',
    'freebsdGuest',
    'mandriva64Guest',
    'mandrivaGuest',
    'netware4Guest',
    'netware5Guest',
    'netware6Guest',
    'nld9Guest',
    'oesGuest',
    'openServer5Guest',
    'openServer6Guest',
    'oracleLinux64Guest',
    'oracleLinuxGuest',
    'os2Guest',
    'other24xLinux64Guest',
    'other24xLinuxGuest',
    'other26xLinux64Guest',
    'other26xLinuxGuest',
    'otherGuest',
    'otherGuest64',
    'otherLinux64Guest',
    'otherLinuxGuest',
    'redhatGuest',
    'rhel2Guest',
    'rhel3_64Guest',
    'rhel3Guest',
    'rhel4_64Guest',
    'rhel4Guest',
    'rhel5_64Guest',
    'rhel5Guest',
    'rhel6_64Guest',
    'rhel6Guest',
    'sjdsGuest',
    'sles10_64Guest',
    'sles10Guest',
    'sles11_64Guest',
    'sles11Guest',
    'sles64Guest',
    'slesGuest',
    'solaris10_64Guest',
    'solaris10Guest',
    'solaris11_64Guest',
    'solaris6Guest',
    'solaris7Guest',
    'solaris8Guest',
    'solaris9Guest',
    'suse64Guest',
    'suseGuest',
    'turboLinux64Guest',
    'turboLinuxGuest',
    'ubuntu64Guest',
    'ubuntuGuest',
    'unixWare7Guest',
    'vmkernel5Guest',
    'vmkernelGuest',
    'win2000AdvServGuest',
    'win2000ProGuest',
    'win2000ServGuest',
    'win31Guest',
    'win95Guest',
    'win98Guest',
    'windows7_64Guest',
    'windows7Guest',
    'windows7Server64Guest',
    'windows8_64Guest',
    'windows8Guest',
    'windows8Server64Guest',
    'winLonghorn64Guest',
    'winLonghornGuest',
    'winMeGuest',
    'winNetBusinessGuest',
    'winNetDatacenter64Guest',
    'winNetDatacenterGuest',
    'winNetEnterprise64Guest',
    'winNetEnterpriseGuest',
    'winNetStandard64Guest',
    'winNetStandardGuest',
    'winNetWebGuest',
    'winNTGuest',
    'winVista64Guest',
    'winVistaGuest',
    'winXPHomeGuest',
    'winXPPro64Guest',
    'winXPProGuest',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def MigrationFault(vim, *args, **kwargs):
    '''Base object type for issues that can occur when reassigning the execution host
    of a virtual machine using migrate or relocate. These issues are typically used
    as argument in the MigrationEvent. When a MigrationFault is used as a value in
    a MigrationEvent, the type of MigrationEvent determines if the issue is a
    warning or an error (for example, MigrationHostWarningEvent or
    MigrationHostErrorEvent). When thrown as an exception, the fault is an
    error.Issues are categorized as errors or warnings according to the following
    criteria:If the virtual machine is powered on:If the virtual machine is powered
    off or suspended:'''

    obj = vim.client.factory.create('{urn:vim25}MigrationFault')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'dynamicProperty', 'dynamicType', 'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def GuestOsDescriptor(vim, *args, **kwargs):
    '''This data object type contains information to describe a particular guest
    operating system.'''

    obj = vim.client.factory.create('{urn:vim25}GuestOsDescriptor')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 30:
        raise IndexError('Expected at least 31 arguments got: %d' % len(args))

    required = [ 'family', 'fullName', 'ich7mRecommended', 'id', 'numSupportedCoresPerSocket',
        'numSupportedPhysicalSockets', 'recommendedColorDepth',
        'recommendedDiskController', 'recommendedDiskSizeMB', 'recommendedFirmware',
        'recommendedMemMB', 'smcRecommended', 'smcRequired',
        'supportedDiskControllerList', 'supportedEthernetCard', 'supportedFirmware',
        'supportedForCreate', 'supportedMaxCPUs', 'supportedMaxMemMB',
        'supportedMinMemMB', 'supportedNumDisks', 'supportLevel', 'supports3D',
        'supportsCpuHotAdd', 'supportsCpuHotRemove', 'supportsMemoryHotAdd',
        'supportsVMI', 'supportsWakeOnLan', 'usbRecommended', 'vRAMSizeInKB' ]
    optional = [ 'cpuFeatureMask', 'recommendedEthernetCard', 'recommendedSCSIController',
        'recommendedUSBController', 'supportedUSBControllerList', 'supportsSlaveDisk',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualMachineConfigOption(vim, *args, **kwargs):
    '''This configuration data object type contains information about the execution
    environment for a virtual machine. This includes information about which
    features are supported, such as:VirtualCenter can provide a broader environment
    than any single physical host. This is a departure from traditional
    virtualization approaches, which rely on the host system to define the
    environment for virtual machines. This data object describes environment
    capabilities and is used by VirtualCenter to choose hosts on which to run
    virtual machines.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualMachineConfigOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 8:
        raise IndexError('Expected at least 9 arguments got: %d' % len(args))

    required = [ 'capabilities', 'datastore', 'description', 'guestOSDefaultIndex',
        'guestOSDescriptor', 'hardwareOptions', 'supportedMonitorType', 'version' ]
    optional = [ 'defaultDevice', 'supportedOvfEnvironmentTransport',
        'supportedOvfInstallTransport', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineTargetInfoConfigurationTag = Enum(
    'clusterWide',
    'compliant',

)




from UserDict import DictMixin
from infi.pyutils.lazy import clear_cache, cached_method
from cjson import encode, decode

MANAGED_OBJECT_REFERENCE = "Folder:group-d1"

class ExtensionResourceDict(object, DictMixin):
    """
    The problem with vSphere is that is does not expose extension's resources.
    For this purpose we have this dict on top of a custom field at the root folder
    """
    def __init__(self, client, field_name):
        super(ExtensionResourceDict, self).__init__()
        self._client = client
        self._field_name = field_name
        self._managed_object = self._client.getManagedObjectByReference(MANAGED_OBJECT_REFERENCE)
        self._manager = self._client.service_content.customFieldsManager
        if not self._is_custom_field_installed():
            self._install_custom_field()

    def keys(self):
        return self._get_dict().keys()

    def __getitem__(self, key):
        return self._get_dict()[key]

    def __setitem__(self, key, value):
        dictionary = self._get_dict()
        dictionary[key] = value
        self._set_dict(dictionary)

    def __delitem__(self, key):
        dictionary = self._get_dict()
        dictionary.__delitem__(key)
        self._set_dict(dictionary)

    def _get_dict(self):
        return decode(self._get_json_string())

    def _set_dict(self, dictionary):
        self._set_json_string(encode(dictionary))

    @cached_method
    def _is_custom_field_installed(self):
        return any([field.name == self._field_name for field in self._manager.field])

    def _install_custom_field(self):
        self._manager.AddCustomFieldDef(name=self._field_name, moType="Folder", fieldDefPolicy=None, fieldPolicy=None)
        clear_cache(self)

    @cached_method
    def _get_custom_field_key(self):
        return [field for field in self._manager.field if field.name == self._field_name][0].key

    @cached_method
    def _get_json_string(self):
        item = [item for item in self._managed_object.customValue if item.key == self._get_custom_field_key()]
        return encode({}) if not item else item[0].value

    def _set_json_string(self, json):
        self._manager.SetField(entity=self._managed_object, key=self._get_custom_field_key(), value=json)
        clear_cache(self)

class ExtensionResourceFactory(object):
    @classmethod
    def get_dict(cls, client, extension_key):
        return ExtensionResourceDict(client, extension_key)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VAppIPAssignmentInfoProtocols = Enum(
    'IPv4',
    'IPv6',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostActiveDirectoryInfoDomainMembershipStatus = Enum(
    'clientTrustBroken',
    'inconsistentTrust',
    'noServers',
    'ok',
    'otherProblem',
    'serverTrustBroken',
    'unknown',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterDasVmConfigInfo(vim, *args, **kwargs):
    '''The ClusterDasVmConfigInfo data object contains the HA configuration for a
    single virtual machine.All fields are optional. If you set the parameter to
    when you call ReconfigureComputeResource_Task, an unset property has no effect
    on the existing property value in the cluster configuration on the Server. If
    you set the parameter to when you reconfigure a cluster, the cluster
    configuration is reverted to the default values, then the new configuration
    values are applied.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterDasVmConfigInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'key' ]
    optional = [ 'dasSettings', 'powerOffOnIsolation', 'restartPriority', 'dynamicProperty',
        'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageFilesystem(Base):
    '''
    Operations pertaining to filesystems, also known as datastores, on the ESX host.
    '''
    moid = 'ha-cli-handler-storage-filesystem'
    def automount(self):
        '''
        Request mounting of known datastores not explicitly unmounted.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.filesystem.Automount',
                            )
    def mount(self, nopersist=None, volumelabel=None, volumeuuid=None):
        '''
        Connect to and mount an unmounted volume on the ESX host.
        :param nopersist: boolean, Mount the volume non-peristently; the volume will not be mounted after a restart.
        :param volumelabel: string, The label of the volume to mount. This volume must be unmounted for this operation to succeed.
        :param volumeuuid: string, The UUID of the VMFS filesystem to mount. This volume must be unmounted for this operation to succeed.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.filesystem.Mount',
                            nopersist=nopersist,
                            volumelabel=volumelabel,
                            volumeuuid=volumeuuid,
                            )
    def unmount(self, nopersist=None, volumelabel=None, volumepath=None, volumeuuid=None):
        '''
        Disconnect and unmount and existing VMFS or NAS volume. This will not delete the configuration for the volume, but will remove the volume from the list of mounted volumes.
        :param nopersist: boolean, Unmount the volume non-peristently; the volume will be automounted after a restart.
        :param volumelabel: string, The label of the volume to unmount.
        :param volumepath: string, The path of the volume to unmount.
        :param volumeuuid: string, The uuid of the volume to unmount.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.filesystem.Unmount',
                            nopersist=nopersist,
                            volumelabel=volumelabel,
                            volumepath=volumepath,
                            volumeuuid=volumeuuid,
                            )
    def list(self):
        '''
        List the volumes available to the host. This includes VMFS, NAS and VFAT partitions.
        :returns: vim.EsxCLI.storage.filesystem.list.FilesystemVolume[]
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.filesystem.List',
                            )
    def rescan(self):
        '''
        Issue a rescan operation to the VMkernel to have is scan storage devices for new mountable filesystems.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.filesystem.Rescan',
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def MultipleCertificatesVerifyFault(vim, *args, **kwargs):
    '''MultipleCertificatesVerifyFault is thrown by the host connect method
    ReconnectHost_Task as well as the methods to add a host to VirtualCenter
    (AddStandaloneHost_Task and AddHost_Task) if VirtualCenter detects that the
    host has different SSL certificates for different management ports. This can
    occur, for example, if an ESX 2.x host has different SSL certificates for the
    authd service (port 902) and the Management UI port (port 443). VirtualCenter
    is not able to manage such hosts. To fix this issue, the user should modify the
    host to ensure there is only one certificate for all services. Alternatively,
    different certificates are allowed as long as each certificate is verifiable
    (trusted) by the VirtualCenter server.'''

    obj = vim.client.factory.create('{urn:vim25}MultipleCertificatesVerifyFault')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 5:
        raise IndexError('Expected at least 6 arguments got: %d' % len(args))

    required = [ 'thumbprintData', 'dynamicProperty', 'dynamicType', 'faultCause',
        'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

HostLicensableResourceKey = Enum(
    'memoryForVms',
    'memorySize',
    'numCpuCores',
    'numCpuPackages',
    'numVmsStarted',
    'numVmsStarting',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CannotAccessVmDevice(vim, *args, **kwargs):
    '''One of the virtual machine's devices uses a backing that is not accessible on
    the host.Following is a discussion of this fault's use in migration
    validation.This is an error if the device is currently connected and a warning
    otherwise. Devices that can be disconnected can only be connected if the
    virtual machine is powered on.The usage of this fault is slightly different if
    the backing of a device is inherently host-local, and therefore not shared or
    globally named among hosts. (Examples of such backings: physical CD-ROM drive,
    physical serial port.) If a device with such a backing is currently connected,
    that will be a migration error. If the device is disconnected, there will be a
    warning if no backing with the same name exists on the destination host. If the
    device is disconnected and a backing with the same name exists on the
    destination host, this is neither a warning nor an error case, even though the
    destination host's backing is not the same instance as the source host's. It is
    assumed that use of the host-local backing is what is desired for the device.'''

    obj = vim.client.factory.create('{urn:vim25}CannotAccessVmDevice')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 7:
        raise IndexError('Expected at least 8 arguments got: %d' % len(args))

    required = [ 'backing', 'connected', 'device', 'dynamicProperty', 'dynamicType',
        'faultCause', 'faultMessage' ]
    optional = [  ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

VirtualMachineNeedSecondaryReason = Enum(
    'divergence',
    'initializing',
    'lostConnection',
    'other',
    'partialHardwareFailure',
    'userAction',

)




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

ScsiLunDescriptorQuality = Enum(
    'highQuality',
    'lowQuality',
    'mediumQuality',
    'unknownQuality',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def CustomizationUserData(vim, *args, **kwargs):
    '''Personal data pertaining to the owner of the virtual machine.The UserData data
    object type maps to the UserData key in the answer file. These values are
    transferred directly into the file that VirtualCenter stores on the target
    virtual disk. For more detailed information, see the document .'''

    obj = vim.client.factory.create('{urn:vim25}CustomizationUserData')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'computerName', 'fullName', 'orgName', 'productId' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostImageConfigManager(BaseEntity):
    '''This managed object is the interface for configuration of the ESX software
    image, including properties such as acceptance level. It is currently designed
    to be host agent specific.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostImageConfigManager):
        super(HostImageConfigManager, self).__init__(core, name=name, ref=ref, type=type)

    

    
    
    def HostImageConfigGetAcceptance(self):
        '''Queries the current host acceptance level setting.See HostImageAcceptanceLevel
        
        '''
        return self.delegate("HostImageConfigGetAcceptance")()
    
    def HostImageConfigGetProfile(self):
        '''Queries the current host image profile information.See HostImageProfileSummary
        
        '''
        return self.delegate("HostImageConfigGetProfile")()
    
    def UpdateHostImageAcceptanceLevel(self, newAcceptanceLevel):
        '''Sets the acceptance level of the host image profile.See
        HostImageAcceptanceLevel
        
        :param newAcceptanceLevel: the new AcceptanceLevel to set.See HostImageAcceptanceLevel
        
        '''
        return self.delegate("UpdateHostImageAcceptanceLevel")(newAcceptanceLevel)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def DVSNameArrayUplinkPortPolicy(vim, *args, **kwargs):
    '''The uplink port policy specifies an array of uniform names for the uplink ports
    across the hosts. The size of the array indicates the number of uplink ports
    that will be created for each host in the switch.When the names in this array
    change, the uplink ports on all the hosts are automatically renamed
    accordingly. Increasing the number of names in the array automatically creates
    additional uplink ports bearing the added name on each host. Decreasing the
    number of name automatically deletes the unused uplink ports on each host.
    Decreasing beyond the number of unused uplink port raises a fault.This policy
    overrides the portgroup's port naming format, if both are defined and the
    uplink ports are created in a uplink portgroup.'''

    obj = vim.client.factory.create('{urn:vim25}DVSNameArrayUplinkPortPolicy')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'uplinkPortName' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class StorageNmpPspRoundrobinDeviceconfig(Base):
    '''
    Round robin PSP device configuration operations
    '''
    moid = 'ha-cli-handler-storage-nmp-psp-roundrobin-deviceconfig'
    def set(self, device, bytes=None, iops=None, type=None, useano=None):
        '''
        Allow setting of the Round Robin path options on a given device controlled by the Round Robin Selection Policy.
        :param bytes: long, When the --type option is set to 'bytes' this is the  value that will be assigned to the byte limit value for this device.
        :param device: string, The device you wish to set the Round Robin settings for. This device must be controlled by the Round Roubin Path Selection Policy
        :param iops: long, When the --type option is set to 'iops' this is the  value that will be assigned to the I/O operation limit value for this device.
        :param type: string, Set the type of the Round Robin path switching that should be enabled for this device. Valid values for type are: 
    bytes: Set the trigger for path switching based on the number of bytes sent down a path.
    default: Set the trigger for path switching back to default values.
    iops: Set the trigger for path switching based on the number of I/O operations on a path.

        :param useano: boolean, Set useano to true,to also include non-optimizedpaths in the set of active paths used to issue I/Os on this device,otherwise set it to false
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.roundrobin.deviceconfig.Set',
                            bytes=bytes,
                            device=device,
                            iops=iops,
                            type=type,
                            useano=useano,
                            )
    def get(self, device):
        '''
        Allow retrieving of Round Robin Path Selection Policy settings for a given device.
        :param device: string, The device you wish to get the Round Robin properties for.
        :returns: vim.EsxCLI.storage.nmp.psp.roundrobin.deviceconfig.get.RoundRobinDeviceConfiguration
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.storage.nmp.psp.roundrobin.deviceconfig.Get',
                            device=device,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterDpmHostConfigSpec(vim, *args, **kwargs):
    '''The ClusterDpmHostConfigSpec data object provides information that the Server
    uses to update the DPM configuration for a single host (identified by the key
    property). The host DPM configuration overrides the cluster default DPM setting
    (ClusterConfigSpecEx.dpmConfig).The vSphere API defines three update operations
    (ArrayUpdateSpec.operation).Use the ReconfigureComputeResource_Task method to
    update the DPM configuration. If you set the modify parameter to true, you can
    use any of the three operations (add, edit, or remove). If you set the modify
    parameter to false, you can use only the add operation.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterDpmHostConfigSpec')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'operation' ]
    optional = [ 'info', 'removeKey', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.mo.extensible_managed_object import ExtensibleManagedObject

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HostMemorySystem(ExtensibleManagedObject):
    '''The MemoryManagerSystem managed object provides an interface through which the
    host memory management policies that affect the performance of running virtual
    machines can be gathered and configured.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HostMemorySystem):
        super(HostMemorySystem, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def consoleReservationInfo(self):
        '''Service console reservation information for the memory manager. The existence
        of this data object indicates if the service console memory reservation must be
        configured for this host.'''
        return self.update('consoleReservationInfo')
    @property
    def virtualMachineReservationInfo(self):
        '''Virtual machine reservation information for the memory manager. The existence
        of this data object indicates if the virtual machine memory reservation must be
        configured for this host.'''
        return self.update('virtualMachineReservationInfo')

    
    
    def ReconfigureServiceConsoleReservation(self, cfgBytes):
        '''Sets the configured service console memory reservation. This change affects
        only the serviceConsoleReservedCfg property. The configuration change
        propagates to the other properties after the next boot.
        
        :param cfgBytes: 
        
        '''
        return self.delegate("ReconfigureServiceConsoleReservation")(cfgBytes)
    
    def ReconfigureVirtualMachineReservation(self, spec):
        '''Updates the virtual machine reservation information.
        
        :param spec: 
        
        '''
        return self.delegate("ReconfigureVirtualMachineReservation")(spec)


'''
Created on Jul 29, 2011

@author: eplaster
'''

class InvalidArgumentError(Exception):
    pass

class PyVisdkError(Exception):
    pass

class VisdkTaskError(Exception):
    pass

class VisdkInvalidState(Exception):
    pass




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

InvalidProfileReferenceHostReason = Enum(
    'incompatibleVersion',
    'missingReferenceHost',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def SendEmailAction(vim, *args, **kwargs):
    '''This data object type defines an email that is triggered by an alarm. You can
    use any elements of the ActionParameter enumerated list as part of your strings
    to provide information available at runtime.'''

    obj = vim.client.factory.create('{urn:vim25}SendEmailAction')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'body', 'ccList', 'subject', 'toList' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualPointingDeviceOption(vim, *args, **kwargs):
    '''The VirtualPointingDeviceOption data object type contains the options for the
    host mouse type defined in the VirtualPointingDevice data object type. These
    options include the valid selections for the mouse type, the supported mouse
    types, and the default mouse type.'''

    obj = vim.client.factory.create('{urn:vim25}VirtualPointingDeviceOption')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'deprecated', 'hotRemoveSupported', 'plugAndPlay', 'type' ]
    optional = [ 'autoAssignController', 'backingOption', 'connectOption', 'controllerType',
        'defaultBackingOptionIndex', 'licensingLimit', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def AuthorizationPrivilege(vim, *args, **kwargs):
    '''This data object type provides access to some aspect of the system. Privileges
    are generally independent. This means a user with a privilege usually can
    perform an associated set of actions without needing any additional supporting
    privileges.Within each product version, privileges do not change. See
    AuthorizationDescription for detailed information on the privileges defined by
    the system.'''

    obj = vim.client.factory.create('{urn:vim25}AuthorizationPrivilege')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 4:
        raise IndexError('Expected at least 5 arguments got: %d' % len(args))

    required = [ 'name', 'onParent', 'privGroupName', 'privId' ]
    optional = [ 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class HttpNfcLease(BaseEntity):
    '''Represents a lease on a VirtualMachine or a VirtualApp, which can be used to
    import or export disks for the entity. While the lease is held, operations that
    alter the state of the virtual machines covered by the lease are blocked.
    Examples of blocked operations are PowerOn, Destroy, Migrate, etc.A lease is in
    one of four states:The import/export task corresponding to the lease continues
    running while the lease is held.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.HttpNfcLease):
        super(HttpNfcLease, self).__init__(core, name=name, ref=ref, type=type)

    
    @property
    def error(self):
        '''If the lease is in the error state, this property contains the error that
        caused the lease to be aborted.'''
        return self.update('error')
    @property
    def info(self):
        '''Provides information on the objects contained in this lease. The info property
        is only valid when the lease is in the ready state.'''
        return self.update('info')
    @property
    def initializeProgress(self):
        '''Provides progress information (0-100 percent) for the initializing state of the
        lease. Clients can use this to track overall progress.'''
        return self.update('initializeProgress')
    @property
    def state(self):
        '''The current state of the lease.'''
        return self.update('state')

    
    
    def HttpNfcLeaseAbort(self, fault=None):
        '''Aborts the import/export and releases this lease. Operations on the objects
        contained in this lease will no longer be blocked. After calling this method,
        this lease will no longer be valid.Aborts the import/export and releases this
        lease. Operations on the objects contained in this lease will no longer be
        blocked. After calling this method, this lease will no longer be valid.
        
        :param fault: [in] The fault that caused the abort, if any.
        
        '''
        return self.delegate("HttpNfcLeaseAbort")(fault)
    
    def HttpNfcLeaseComplete(self):
        '''Completes the import/export and releases this lease. Operations on the objects
        contained in this lease will no longer be blocked. After calling this method,
        this lease will no longer be valid.Completes the import/export and releases
        this lease. Operations on the objects contained in this lease will no longer be
        blocked. After calling this method, this lease will no longer be valid.
        
        '''
        return self.delegate("HttpNfcLeaseComplete")()
    
    def HttpNfcLeaseGetManifest(self):
        '''Gets the download manifest for this lease.
        
        '''
        return self.delegate("HttpNfcLeaseGetManifest")()
    
    def HttpNfcLeaseProgress(self, percent):
        '''Sets the disk up/download progress, and renews this lease. A lease will time
        out automatically after a while. If the client wishes to continue using it, for
        example if it is not done accessing the disks, this method must be called
        periodically.
        
        :param percent: [in] Completion status represented as an integer in the 0-100 range.
        
        '''
        return self.delegate("HttpNfcLeaseProgress")(percent)



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def PerformanceManagerCounterLevelMapping(vim, *args, **kwargs):
    '''PerformanceManagerCounterLevelMapping class captures the counter and level
    mapping. Any counter has two aspects: aggregate value or the per-device value.
    For example, cpu.usage.average on a host is an aggregate counter and
    cpu.usage.average on pcpus in a host is a per-device counters. There is a need
    to be able to specify different levels for the two versions. Currently, all
    per-device stats are collected at level greater than or equal to 3. In order to
    be able to configure the level of collections for aggregate and per-device
    counters we have two optional level fields.
    PerformanceManagerCounterLevelMapping is used to update the levels for a
    counter.'''

    obj = vim.client.factory.create('{urn:vim25}PerformanceManagerCounterLevelMapping')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 1:
        raise IndexError('Expected at least 2 arguments got: %d' % len(args))

    required = [ 'counterId' ]
    optional = [ 'aggregateLevel', 'perDeviceLevel', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.base.managed_object_types import ManagedObjectTypes

from pyvisdk.base.base_entity import BaseEntity

import logging

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

class VirtualizationManager(BaseEntity):
    '''The VirtualizationManager is the interface for discover and consolidate host
    and services from physical environment to virtualization environment.'''

    def __init__(self, core, name=None, ref=None, type=ManagedObjectTypes.VirtualizationManager):
        super(VirtualizationManager, self).__init__(core, name=name, ref=ref, type=type)

    

    



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def Capability(vim, *args, **kwargs):
    '''A particular product may or may not support certain features. This data object
    indicates whether or not a service instance implements these features. This
    data object type indicates the circumstances under which an operation throws a
    NotSupported fault.Support for some features is indicated by the presence or
    absence of the manager object from the service instance. For example, the
    AlarmManager manager object indicates collecting alarms is supported. Other
    features indicate whether or not a given operation on an object throws a
    NotSupported fault.Some capabilities depend on the host or virtual machine
    version. These are specified by using the vim.host.Capability and
    vim.vm.Capability objects.'''

    obj = vim.client.factory.create('{urn:vim25}Capability')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 3:
        raise IndexError('Expected at least 4 arguments got: %d' % len(args))

    required = [ 'multiHostSupported', 'provisioningSupported', 'userShellAccessSupported' ]
    optional = [ 'supportedEVCMode', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VAppIPAssignmentInfo(vim, *args, **kwargs):
    '''The IPAssignmentInfo class specifies how the guest software gets configured
    with IP addresses, including protocol type (IPv4 or IPv6) and the life-time of
    those IP addresses.A vApp/virtual machine can either use DHCP to acquire an IP
    configuration, or it can acquire its IP configuration through the use of the VI
    platform using the OVF environment's properties. The latter is a known as OVF-
    environment-assigned IP configuration.Guest software can be constructed to
    support DHCP , OVF assigned IP configuration, or both. The
    supportedAssignmentScheme property lists the supported schemes. This is
    typically specified by the author of a vApp.The deployer / operator of a vApp,
    specifies what IP allocation policy should be used:Transient and fixed
    assignment differs in the life time of the IP allocation. For transient, IP
    addresses are automatically assigned on power-on and released on power-off. For
    a fixed, the IP addresses are explicitly specified by the deployer and does not
    change between a power-on/power-off.The IPAssignment settings are global to a
    deployment. Thus,if a vApp or virtual machine is part of another vApp, then the
    settings are ignored, and the ones for the top-most vApp container is used.'''

    obj = vim.client.factory.create('{urn:vim25}VAppIPAssignmentInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'ipAllocationPolicy', 'ipProtocol', 'supportedAllocationScheme',
        'supportedIpProtocol', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class NetworkVswitchStandardPolicySecurity(Base):
    '''
    Commands to manipulate security policy setting for the given virtual switch.
    '''
    moid = 'ha-cli-handler-network-vswitch-standard-policy-security'
    def set(self, vswitchname, allowforgedtransmits=None, allowmacchange=None, allowpromiscuous=None):
        '''
        Set the security policy for a given virtual switch
        :param allowforgedtransmits: boolean, Allow ports on the virtual switch to send packets with forged source information.
        :param allowmacchange: boolean, Allow ports on the virtual switch to change their MAC address.
        :param allowpromiscuous: boolean, Allow ports on the virtual switch to enter promiscuous mode.
        :param vswitchname: string, The name of the virtual switch to use when setting the switch security policy.
        :returns: boolean
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.policy.security.Set',
                            allowforgedtransmits=allowforgedtransmits,
                            allowmacchange=allowmacchange,
                            allowpromiscuous=allowpromiscuous,
                            vswitchname=vswitchname,
                            )
    def get(self, vswitchname):
        '''
        Get the Security Policy governing the given virtual switch.
        :param vswitchname: string, The name of the virtual switch to use when fetching the network security policy.
        :returns: vim.EsxCLI.network.vswitch.standard.policy.security.get.VirtualSwitchSecurityPolicy
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.network.vswitch.standard.policy.security.Get',
                            vswitchname=vswitchname,
                            )   



import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def StorageIOAllocationInfo(vim, *args, **kwargs):
    '''The IOAllocationInfo specifies the shares and the limit for storage I/O
    resource.The storage I/O resource is allocated to virtual machines based on
    their shares and limit. We do not support reservation for storage I/O resource.
    And we do not support storage I/O resource management on resource pools.Each
    virtual machine has one IOAllocationInfo object per virtual disk. For example,
    we can specify that a virtual machine has 500 shares on the first virtual disk,
    1000 shares on the second virtual disk, etc.'''

    obj = vim.client.factory.create('{urn:vim25}StorageIOAllocationInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'limit', 'shares', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

DistributedVirtualSwitchHostMemberHostComponentState = Enum(
    'disconnected',
    'down',
    'outOfSync',
    'pending',
    'up',
    'warning',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def VirtualSerialPortURIBackingInfo(vim, *args, **kwargs):
    '''The data object specifies network backing for a . You can use URI backing to
    create a network serial port on the virtual machine, supporting connections
    between the virtual machine and remote systems.When a virtual machine
    establishes a connection with a remote system on the network, the virtual
    machine can act as a server or a client. When the virtual machine acts as a
    server, it accepts a connection. When the virtual machine acts as a client, it
    initiates the connection.You can configure the virtual serial port for
    communication through a virtual serial port concentrator that acts as a proxy
    between the virtual machine and the network. When you specify a , the virtual
    machine initiates the connection with the concentrator and forwards the and to
    the concentrator. For information about using a virtual serial port
    concentrator, see .ESX hosts support different protocols depending on your
    virtual serial port configuration.You cannot specify a username and password in
    the proxy URI. Any text following the port specification is ignored.For
    information about URI format, see .'''

    obj = vim.client.factory.create('{urn:vim25}VirtualSerialPortURIBackingInfo')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 2:
        raise IndexError('Expected at least 3 arguments got: %d' % len(args))

    required = [ 'direction', 'serviceURI' ]
    optional = [ 'proxyURI', 'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




########################################
# Automatically generated, do not edit.
########################################


from pyvisdk.thirdparty import Enum

SlpDiscoveryMethod = Enum(
    'slpAutoMulticast',
    'slpAutoUnicast',
    'slpDhcp',
    'slpManual',

)




import logging
from pyvisdk.exceptions import InvalidArgumentError

########################################
# Automatically generated, do not edit.
########################################

log = logging.getLogger(__name__)

def ClusterConfigSpecEx(vim, *args, **kwargs):
    '''The ClusterConfigSpecEx data object provides a set of update specifications for
    complete cluster configuration. You can configure a cluster when you create a
    new cluster (the CreateClusterEx method) or when you reconfigure an existing
    cluster (the ReconfigureComputeResource_Task method).All fields are optional.
    If you set the parameter to when you call ReconfigureComputeResource_Task, an
    unset property has no effect on the existing property value in the cluster
    configuration on the Server. If you set the parameter to when you reconfigure a
    cluster, the cluster configuration is reverted to the default values, then the
    new configuration values are applied.Use the properties defined for this object
    to configure the following services:When it chooses a failover host, HA selects
    a host that is compatible with the virtual machine and that can support
    resource allocation for that virtual machine so that service level guarantees
    remain intact. HA does not consider hosts that are in maintenance mode, standby
    mode, or which are disconnected from the vCenter Server. When a host powers on
    or becomes available again, HA is reenabled on that host, so it becomes
    available for failover again. VMware recommends that you configure hosts and
    virtual machines so that all virtual machines can run on all hosts in the
    cluster. This will maximize the chances of restarting a VM after a failure.HA
    also restarts a virtual machine after a guest operating system failure. In this
    case, the virtual machine health monitoring service detects the guest failure,
    and HA restarts the virtual machine on the same host. The service monitors
    heartbeats from the VmTools service and optionally heartbeats that are
    generated by a third-party application monitor. See
    ClusterVmToolsMonitoringSettings and ClusterDasConfigInfo.vmMonitoring.To
    enable HA for a cluster, set the ClusterDasConfigInfo.enabled property to and
    the ClusterDasConfigInfo.hostMonitoring property to enabled. (The vSphere API
    uses the substring "das" in object, property, and method names for HA. )To
    enable DRS for a cluster, set the ClusterDrsConfigInfo.enabled property to .To
    enable DPM for a cluster, set the ClusterDpmConfigInfo.enabled property to .The
    HA, DRS, and DPM services are integrated with the FT (Fault Tolerance) and EVC
    (Enhanced vMotion Compatibility) services. Use the CreateSecondaryVM_Task
    method to establish fault tolerance for a virtual machine. Use the vSphere
    Client to configure EVC. The HA, DRS, DPM, FT, and EVC services interact under
    the following circumstances.When admission control is disabled, failover
    resource constraints are not passed on to DRS and DPM. The constraints are not
    enforced.If EVC is disabled, vCenter automatically creates overrides to disable
    DRS for FT primary/secondary pairs in the cluster. vCenter will still use DRS
    to place a secondary virtual machine when it powers on. Attempts to remove the
    overrides or to enable DRS operations will fail.High Availability was
    previously called Distributed Availability Services.'''

    obj = vim.client.factory.create('{urn:vim25}ClusterConfigSpecEx')

    # do some validation checking...
    if (len(args) + len(kwargs)) < 0:
        raise IndexError('Expected at least 1 arguments got: %d' % len(args))

    required = [  ]
    optional = [ 'dasConfig', 'dasVmConfigSpec', 'dpmConfig', 'dpmHostConfigSpec', 'drsConfig',
        'drsVmConfigSpec', 'groupSpec', 'rulesSpec', 'spbmEnabled', 'vmSwapPlacement',
        'dynamicProperty', 'dynamicType' ]

    for name, arg in zip(required+optional, args):
        setattr(obj, name, arg)

    for name, value in kwargs.items():
        if name in required + optional:
            setattr(obj, name, value)
        else:
            raise InvalidArgumentError("Invalid argument: %s.  Expected one of %s" % (name, ", ".join(required + optional)))

    return obj




from pyvisdk.esxcli.executer import execute_soap
from pyvisdk.esxcli.base import Base

class SystemProcessStatsRunning(Base):
    '''
    Number of currently running worlds
    '''
    moid = 'ha-cli-handler-system-process-stats-running'
    def get(self):
        '''
        Number of currently running processes.
        :returns: vim.EsxCLI.system.process.stats.running.get.ProcessStatsRunningGet
        '''
        return execute_soap(self._client, self._host, self.moid, 'vim.EsxCLI.system.process.stats.running.Get',
                            )   